"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Platform = void 0;
const discovery_1 = require("@watsonb8/wiz-lib/build/discovery");
const node_hue_api_1 = require("node-hue-api");
const fluxAccessory_1 = require("./fluxAccessory");
const settings_1 = require("./settings");
const sleep_1 = require("./sleep");
class Platform {
    log;
    api;
    hue;
    wiz;
    accessory = undefined;
    config;
    constructor(log, config, api) {
        this.log = log;
        this.api = api;
        this.config = config;
        this.wiz = new discovery_1.Discover();
        this.log.info("INFO - Registering Flux platform");
        this.api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
    }
    async configureAccessory(accessory) {
        this.log.info("Loading accessory from cache:", accessory.displayName);
        this.accessory = accessory;
    }
    /**
     * Handler for didFinishLaunching
     * Happens after constructor
     */
    async didFinishLaunching() {
        this.log.info(`INFO - Done registering Flux platform`);
        await this.connectHue();
        const wizBulbs = await this.connectWiz();
        this.log.info("Registering accessory: " + this.config.name);
        const uuid = this.api.hap.uuid.generate(this.config.name);
        // Load accessory if not cached
        if (!this.accessory) {
            this.accessory = new this.api.platformAccessory(this.config.name, uuid);
            this.accessory.context["DeviceName"] = this.config.name;
            this.accessory.context["Type"] = typeof "Flux";
            this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [
                this.accessory,
            ]);
        }
        new fluxAccessory_1.FluxAccessory({
            platform: this,
            accessory: this.accessory,
            hue: this.hue,
            wizBulbs: wizBulbs ?? [],
            config: this.config,
        });
    }
    connectWiz = async () => {
        if (!this.config) {
            return;
        }
        let bulbs = await this.wiz.createWizBulbs(this.config.wizLights);
        if (this.config.wizDiscoveryEnabled) {
            let discoveredBulbs = await this.wiz.discover();
            let filtered = [];
            for (const bulb of discoveredBulbs) {
                if (!bulbs.some((manualBulb) => manualBulb.getIp() === bulb.getIp()) &&
                    bulb.isRGB()) {
                    filtered.push(bulb);
                }
            }
            bulbs.push(...filtered);
        }
        return bulbs;
    };
    connectHue = async () => {
        if (!this.config) {
            return;
        }
        if (this.config.userName && this.config.clientKey) {
            this.hue = await node_hue_api_1.v3.api
                .createLocal(this.config.ipAddress)
                .connect(this.config.userName, this.config.clientKey, undefined);
            this.log.info("Using existing connection info");
        }
        else {
            const unauthenticatedApi = await node_hue_api_1.v3.api
                .createLocal(this.config.ipAddress)
                .connect(undefined, undefined, undefined);
            let createdUser;
            let connected = false;
            while (!connected) {
                try {
                    this.log.info("Creating hue user. Push link button");
                    createdUser = await unauthenticatedApi.users.createUser("homebridge", "HueChase");
                    this.hue = await node_hue_api_1.v3.api
                        .createLocal(this.config.ipAddress)
                        .connect(createdUser.username, createdUser.clientKey, undefined);
                    this.log.info("Connected to Hue Bridge");
                    this.log.info(`UserName: ${createdUser.username}, ClientKey: ${createdUser.clientkey}`);
                    connected = true;
                }
                catch (err) {
                    if (err.getHueErrorType() === 101) {
                        this.log.info("The Link button on the bridge was not pressed. Please press the Link button and try again.");
                        (0, sleep_1.Sleep)(5000);
                    }
                    else {
                        this.log.info(`Unexpected Error: ${err.message}`);
                        break;
                    }
                }
            }
        }
    };
}
exports.Platform = Platform;
//# sourceMappingURL=platform.js.map